﻿using ProjectTrolleybus.Drawnings;
using ProjectTrolleybus.Entities;

namespace ProjectTrolleybus;

/// <summary>
/// Форма конфигурации объекта
/// </summary>
public partial class FormTrolleyBConfig : Form
{
    /// <summary>
    /// Объект - прорисовка автомобиля
    /// </summary>
    private DrawningTrolleyB? _trolleyB;


    /// <summary>
    /// Событие для передачи объекта
    /// </summary>
    private event Action<DrawningTrolleyB> TrolleyBDelegate;

    /// <summary>
    /// Конструктор
    /// </summary>
    public FormTrolleyBConfig()
    {
        InitializeComponent();
        panelRed.MouseDown += Panel_MouseDown;
        panelGreen.MouseDown += Panel_MouseDown;
        panelBlue.MouseDown += Panel_MouseDown;
        panelYellow.MouseDown += Panel_MouseDown;
        panelWhite.MouseDown += Panel_MouseDown;
        panelGray.MouseDown += Panel_MouseDown;
        panelBlack.MouseDown += Panel_MouseDown;
        panelPurple.MouseDown += Panel_MouseDown;

        buttonCancel.Click += (s, e) => Close();


    }

    /// <summary>
    /// Привязка внешнего метода к событию
    /// </summary>
    /// <param name="trolleyBDelegate"></param>
    public void AddEvent(Action<DrawningTrolleyB> trolleyBDelegate)
    {
        TrolleyBDelegate += trolleyBDelegate;
    }

    /// <summary>
    /// Прорисовка объекта
    /// </summary>
    private void DrawObject()
    {
        Bitmap bmp = new(pictureBoxObject.Width, pictureBoxObject.Height);
        Graphics gr = Graphics.FromImage(bmp);
        _trolleyB?.SetPictureSize(pictureBoxObject.Width, pictureBoxObject.Height);
        _trolleyB?.SetPosition(5, 5, Width, Height);
        _trolleyB?.DrawTransport(gr);
        pictureBoxObject.Image = bmp;
    }

    /// <summary>
    /// Передаем информацию при нажатии на Label
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void labelObject_MouseDown(object sender, MouseEventArgs e)
    {
        (sender as Label)?.DoDragDrop((sender as Label)?.Name ?? string.Empty, DragDropEffects.Move | DragDropEffects.Copy);
    }

    /// <summary>
    /// Проверка получаемой информации (ее типа на соответствие требуемому)
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void PanelObject_DragEnter(object sender, DragEventArgs e)
    {
        e.Effect = e.Data?.GetDataPresent(DataFormats.Text) ?? false ? DragDropEffects.Copy : DragDropEffects.None;
    }

    /// <summary>
    /// Действие при приеме перетаскиваемой информации
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void PanelObject_DragDrop(object sender, DragEventArgs e)
    {
        switch (e.Data?.GetData(DataFormats.Text)?.ToString())
        {
            case "labelSimpleObject":
                _trolleyB = new DrawningTrolleyB((int)numericUpDownSpeed.Value, (double)numericUpDownWeight.Value, Color.White);
                break;
            case "labelModifiedObject":
                _trolleyB = new DrawningTrolleybus((int)numericUpDownSpeed.Value, (double)numericUpDownWeight.Value, Color.White,
                    Color.Black, checkBoxBatteryCompartment.Checked, checkBoxHorns.Checked);
                break;
        }

        DrawObject();
    }

    /// <summary>
    /// Передаем информацию при нажатии на Panel
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void Panel_MouseDown(object? sender, MouseEventArgs e)
    {
        (sender as Panel)?.DoDragDrop((sender as Panel)?.BackColor, DragDropEffects.Move | DragDropEffects.Copy);
    }

    private void labelColor_DragEnter(object sender, DragEventArgs e)
    {
        if (e.Data?.GetDataPresent(typeof(Color)) ?? false)
        {
            e.Effect = DragDropEffects.Copy;
        }
        else
        {
            e.Effect = DragDropEffects.None;
        }
    }

    private void labelColor_DragDrop(object sender, DragEventArgs e)
    {
        if (_trolleyB == null)
        {
            return;
        }

        switch (((Label)sender).Name)
        {
            case "labelBodyColor":
                _trolleyB.EntityTrolleyB.setBodyColor((Color)e.Data.GetData(typeof(Color)));
                break;
            case "labelAdditionalColor":
                if (_trolleyB is not DrawningTrolleybus)
                {
                    return;
                }
                (_trolleyB.EntityTrolleyB as EntityTrolleybus).setAdditionalColor((Color)e.Data.GetData(typeof(Color)));
                break;
        }
        DrawObject();
    }

    /// <summary>
    /// Передача объекта
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonAdd_Click(object sender, EventArgs e)
    {
        if (_trolleyB != null)
        {
            TrolleyBDelegate?.Invoke(_trolleyB);
            Close();
        }
    }
}
