﻿using ProjectTrolleybus.Drawnings;
using ProjectTrolleybus.Entities;
using ProjectTrolleybus.MovementStrategy;

namespace ProjectTrolleybus;

/// <summary>
/// Форма работы с объектом "Троллейбус"
/// </summary>
public partial class FormTrolleybus : Form
{
    /// <summary>
    /// Поле-объект для прорисовки объекта
    /// </summary>
    private DrawningTrolleyB? _drawningTrolleyB;

    /// <summary>
    /// Стратегия перемещения
    /// </summary>
    private AbstractStrategy? _strategy;

    public DrawningTrolleyB SetTrolleyB
    {
        set
        {
            _drawningTrolleyB = value;
            _drawningTrolleyB.SetPictureSize(pictureBoxTrolleybus.Width, pictureBoxTrolleybus.Height);
            comboBoxStrategy.Enabled = true;
            _strategy = null;
            Draw();
        }
    }

    /// <summary>
    /// Конструктор формы
    /// </summary>
    public FormTrolleybus()
    {
        InitializeComponent();
        _strategy = null;
    }

    /// <summary>
    /// Метод прорисовки транспорта
    /// </summary>
    private void Draw()
    {
        Bitmap bmp = new(pictureBoxTrolleybus.Width, pictureBoxTrolleybus.Height);
        Graphics gr = Graphics.FromImage(bmp);
        _drawningTrolleyB.DrawTransport(gr);
        pictureBoxTrolleybus.Image = bmp;
    }

    /// <summary>
    /// Перемещение объекта по кнопке (нажатие кнопок навигации)
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    private void ButtonMove_Click(object sender, EventArgs e)
    {
        if (_drawningTrolleyB == null)
        {
            return;
        }

        string name = ((Button)sender)?.Name ?? string.Empty;
        bool result = false;
        switch (name)
        {
            case "buttonUp":
                result = _drawningTrolleyB.MoveVehicle(DirectionType.Up);
                break;
            case "buttonDown":
                result = _drawningTrolleyB.MoveVehicle(DirectionType.Down);
                break;
            case "buttonLeft":
                result = _drawningTrolleyB.MoveVehicle(DirectionType.Left);
                break;
            case "buttonRight":
                result = _drawningTrolleyB.MoveVehicle(DirectionType.Right);
                break;
        }

        if (result)
        {
            Draw();
        }
    }

    private void buttonStrategyStep_Click(object sender, EventArgs e)
    {
        if (_drawningTrolleyB == null)
        {
            return;
        }

        if (comboBoxStrategy.Enabled)
        {
            _strategy = comboBoxStrategy.SelectedIndex switch
            {
                0 => new MoveToCenter(),
                1 => new MoveToBorder(),
                _ => null,
            };
            if (_strategy == null)
            {
                return;
            }
            _strategy.SetData(new MoveableTrolleyB(_drawningTrolleyB), pictureBoxTrolleybus.Width, pictureBoxTrolleybus.Height);
        }

        if (_strategy == null)
        {
            return;
        }

        comboBoxStrategy.Enabled = false;
        _strategy.MakeStep();
        Draw();

        if (_strategy.GetStatus() == StrategyStatus.Finish)
        {
            comboBoxStrategy.Enabled = true;
            _strategy = null;
        }
    }
}